package it.uniroma1.dis.jasmine.ui.utils;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.text.JTextComponent;

/**
 * A popup for text components, has the most common actions for a text: clear, copy, cut, paste, select.
 * The text component is responsible for showing this popup.
 * @author Constantin
 */
public class TextComponentPopup extends JPopupMenu implements ActionListener, PopupMenuListener {
	/**
	 * 
	 */
	private static final long serialVersionUID = -6632175815632539659L;

	/**
	 * This enumeration contains all action commands for a text component popup.
	 * @author Constantin
	 */
	protected static enum TextActionEnum {
		COPY, PASTE, SELECT, CUT, CLEAR;
	}
	
	private final JTextComponent text;
	private final JMenuItem clear, copy, cut, paste, select;
	
	/**
	 * Constructs a popup on a text component.
	 * @param text a valid JTextComponent
	 */
	public TextComponentPopup(JTextComponent text) {
		this.text = text;
		this.addPopupMenuListener(this);
		
		cut = add("Cut");
		cut.setMnemonic(KeyEvent.VK_C);
		cut.setActionCommand(TextActionEnum.CUT.toString());
		cut.addActionListener(this);
		copy = add("Copy");
		copy.setMnemonic(KeyEvent.VK_C);
		copy.setActionCommand(TextActionEnum.COPY.toString());
		copy.addActionListener(this);
		paste = add("Paste");
		paste.setMnemonic(KeyEvent.VK_P);
		paste.setActionCommand(TextActionEnum.PASTE.toString());
		paste.addActionListener(this);
		clear = add("Clear");
		clear.setMnemonic(KeyEvent.VK_L);
		clear.setActionCommand(TextActionEnum.CLEAR.toString());
		clear.addActionListener(this);
		addSeparator();
		select = add("Select all");
		select.setMnemonic(KeyEvent.VK_S);
		select.setActionCommand(TextActionEnum.SELECT.toString());
		select.addActionListener(this);
	}
	
	/**
	 * Defines actions for the popup items.
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	public void actionPerformed(ActionEvent ae) {
		if (text == null)
			return;
		
		TextActionEnum en = TextActionEnum.valueOf(ae.getActionCommand());
		switch (en) {
			case COPY:
				text.copy();
				break;
			case PASTE:
				text.paste();
				break;
			case CUT:
				text.cut();
				break;
			case SELECT:
				text.select(0, text.getText().length());
				break;
			case CLEAR:
				text.replaceSelection("");
				break;
		}
	}
	
	/**
	 * @see javax.swing.event.PopupMenuListener#popupMenuCanceled(javax.swing.event.PopupMenuEvent)
	 */
	public void popupMenuCanceled(PopupMenuEvent pme) {
	}
	
	/**
	 * @see javax.swing.event.PopupMenuListener#popupMenuWillBecomeInvisible(javax.swing.event.PopupMenuEvent)
	 */
	public void popupMenuWillBecomeInvisible(PopupMenuEvent pme) {
	}
	
	/**
	 * On notification enables or disables some of the popup items if the text component has some text selected or not.
	 * @see javax.swing.event.PopupMenuListener#popupMenuWillBecomeVisible(javax.swing.event.PopupMenuEvent)
	 */
	public void popupMenuWillBecomeVisible(PopupMenuEvent pme) {
		if ((pme.getSource() != this) || (text == null))
			return;
		
		if (text.getSelectedText() == null)
			setItemsEnabled(false);
		else
			setItemsEnabled(true);
	}
	
	/**
	 * Enables or disables some of the popup items according to the status of the text component.
	 * @param enabled
	 */
	private void setItemsEnabled(boolean enabled) {
		if (text.isEditable()) {
			clear.setEnabled(enabled);
			copy.setEnabled(enabled);
			cut.setEnabled(enabled);
		} else {
			clear.setEnabled(false);
			copy.setEnabled(enabled);
			cut.setEnabled(false);
			paste.setEnabled(false);
		}
	}
}
